//  
//   Rebex Sample Code License
// 
//   Copyright 2024, REBEX CR s.r.o.
//   All rights reserved.
//   https://www.rebex.net/
// 
//   Permission to use, copy, modify, and/or distribute this software for any
//   purpose with or without fee is hereby granted.
// 
//   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
//   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//   NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
//   HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
//   WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//   OTHER DEALINGS IN THE SOFTWARE.
// 

using System;
using System.Text;
using System.IO;
using System.Threading;
using Rebex.IO;
using Rebex.Net;

namespace Rebex.Samples
{
    /// <summary>
    /// This sample is a simple utility to upload/download a file to/from
    /// a SFTP server using asynchronous methods of the Sftp class.
    /// </summary>
    class GetPutAsync
    {
        [STAThread]
        static void Main (string[] args)
        {
            // set the trial license key
            Rebex.Licensing.Key = Rebex.Samples.TrialKey.Key;


            string method = null;
            if (args.Length == 4)
            {
                switch (args[0].ToLower())
                {
                    case "get": method = "get"; break;
                    case "put": method = "put"; break;
                }
            }

            if (method == null)
            {
                // display syntax if wrong number of parameters or unsupported operation
                Console.WriteLine ("GetPutAsync - uploads or downloads a file from a SFTP server");
                Console.WriteLine ("Syntax: GetPutAsync get|put hostname remotepath localpath");
                Console.WriteLine ("Example: GetPutAsync get test.rebex.net /pub/example/readme.txt readme.txt");
                return;
            }
            string hostname = args[1];
            string remotePath = args[2];
            string localPath = args[3];

            // create Sftp object and set event handlers
            Sftp ftp = new Sftp();
            ftp.TransferProgressChanged += TransferProgressChanged;
            ftp.ResponseRead += ResponseRead;
            ftp.CommandSent += CommandSent;

            try
            {
                // connect to the server
                ftp.Connect (hostname);
                // ask for username and password and login
                Console.Write ("Username: ");
                string username = Console.ReadLine();
                Console.Write ("Password: ");
                string password = Console.ReadLine();
                ftp.Login (username, password);
                Console.WriteLine ();

                // begin asynchronous transfer
                IAsyncResult asyncResult;
                if (method == "get")
                    asyncResult = ftp.BeginGetFile (remotePath, localPath, new AsyncCallback(MyCallback), null);
                else
                    asyncResult = ftp.BeginPutFile (localPath, remotePath, new AsyncCallback(MyCallback), null);

                // do something else here...

                // wait for the transfer to end
                asyncResult.AsyncWaitHandle.WaitOne();

                // get the result
                long bytes;
                if (method == "get")
                    bytes = ftp.EndGetFile (asyncResult);
                else
                    bytes = ftp.EndPutFile (asyncResult);
                // disconnect from the server
                ftp.Disconnect();
            }
            catch (SftpException e)
            {
                // output error description to the console
                Console.WriteLine ("'{0}' error: {1}", e.Status, e);
            }
            finally
            {
                // release all resources
                ftp.Dispose();
            }
        }
        public static void MyCallback (IAsyncResult asyncResult)
        {
            Console.WriteLine ("\nTransfer finished.");
        }

        public static void TransferProgressChanged (object sender, SftpTransferProgressChangedEventArgs e)
        {
            if (e.TransferState == TransferProgressState.DataBlockProcessed)
                Console.WriteLine ("\rTransferred {0} bytes...", e.BytesTransferred);
        }

        public static void ResponseRead (object sender, SftpResponseReadEventArgs e)
        {
            Console.WriteLine (e.Response);
        }

        public static void CommandSent (object sender, SftpCommandSentEventArgs e)
        {
            Console.WriteLine ("-> " + e.Command);
        }

    }
}
